-- thanks to sweetman for contributing parts of the race store code
races = {}
racescount = 0
currentrace = ""
startCheckPointName = ""
checkPointName = ""
cache_file = getSetting("Cache Path") .. "race_scores_rdes.dat"
log("race system using scores file: "..cache_file)

function writeScore(racename,score)
	local f = io.open(cache_file, "a+")
	if not f == nil then
		f:write(racename.."|"..score.."\n")
		f:close()
		log('race score file written')
	end
end

function findScore(racename)
	local f = io.open(cache_file, "r")
	minval = 9999
	if not f == nil then
		for line in f:lines()
		do
			line_ar = explode('|', line)
			if line_ar[1] == racename or tonumber(line_ar[2]) < minval then
				minval = tonumber(rcdt[2])
			end
		end
		log('race score file read')
	else
		log('race score file not existant')
	end
	return minval
end

function loadCheckpoints(racename, positions)
	for a,v in ipairs(positions) do
		if a == 1 then
			oname = startCheckPointName
		else
			oname = checkPointName
		end
		if a == table.getn(positions) and not loop then
			-- put the finish thing at the end, not a normal checkpoint, only on non-loop races
			oname = startCheckPointName
		end
		iname = 'race_'..racename..'_'..a

		if a > 1 then
			log("*creating object '"..oname.."' with name '"..iname.."'.")
			spawnObject(oname, iname, v[1],v[2],v[3], v[4],v[5],v[6], raceEventHandler)
		end
	end
end


	--SetupRace has several parameters:
	--  racename to show on screen (no spaces nor underscore)
	--  array of checkpoints positions
	--  Is loop race ?
	--  Name of the object that will be used as start Checkpoint for the race
	--  Name of the object that will be used as normal Checkpoint race
	--  Print Altitude of the next checpoint on screen ?
function setupRace(racename, positions, loop, startcheck, normalcheck, showaltitude)
	startCheckPointName = startcheck
	checkPointName = normalcheck
	
	for a,v in ipairs(positions) do
		if a == 1 then
			oname = startCheckPointName
		else
			oname = checkPointName
		end
		if a == table.getn(positions) and not loop then
			-- put the finish thing at the end, not a normal checkpoint, only on non-loop races
			oname = startCheckPointName
		end
		iname = 'race_'..racename..'_'..a

		if a == 1 then
			log("*creating object '"..oname.."' with name '"..iname.."'.")					
			spawnObject(oname, iname, v[1],v[2],v[3], v[4],v[5],v[6], raceEventHandler)
		end
		
	end
	-- add the race finally
	races[racename] = {}
	races[racename]['besttime'] = findScore(racename)
	races[racename]['racename'] = racename
	races[racename]['positions'] = positions
	races[racename]['isloop'] = loop
	races[racename]['startTime'] = 0
	races[racename]['lastcheckpoint'] = 1
	races[racename]['checkpointspassed'] = 1
	races[racename]['numcheckpoints'] = table.getn(positions)
	races[racename]['checkpointsloaded'] = false
	races[racename]['showaltitude'] = showaltitude
	if loop then
		races[racename]['goalnum'] = 1
	else
		races[racename]['goalnum'] = table.getn(positions)
	end
	
end

function getCheckPointFromName(instance)
	ar = explode('_', instance)
	return tonumber(ar[3])
end

function getRacenameFromName(instance)
	ar = explode('_', instance)
	return tostring(ar[2])
end

function setupArrow(position)
	--log("reposition arrow: "..position)
	txt = currentrace.."\ncheckpoint "..tostring(position).." / "..races[currentrace]['numcheckpoints']
	if position == 1 and races[currentrace]['isloop'] then
		txt = "Finish"
	end
	if position == -1 then
		-- hide it
		pointArrow(0, 0, 0, "")
	else
		data = races[currentrace]['positions'][position]
		if data then
			if races[currentrace]["showaltitude"] then
				txt = txt .. "\n\nAlt "..string.format("%.0f", races[currentrace]['positions'][position][2] * 3,2808399 ).." feets" 
			end
			pointArrow(data[1], data[2], data[3], txt)
		else
			pointArrow(0, 0, 0, "")
		end
	end
end

function raceEventHandler(instance, box)
	--log("racehandler: "..instance.." - "..box)
	checknum = getCheckPointFromName(instance)
	racename = getRacenameFromName(instance)
	
	log("just passed checkpoint number "..checknum.." of race "..racename)

	-- create an array with all required names in it
	if currentrace == "" then
		-- not racing currently
		if checknum == 1 then
			-- start of the race
			currentrace = racename
			--log(races[currentrace]['racename'])
			races[currentrace]['startTime'] = getTime()
			races[racename]['lastcheckpoint'] = 1
			races[currentrace]['checkpointspassed'] = 1
			setupArrow(2)
			startTimer()
			if not races[racename]['checkpointsloaded'] then
				loadCheckpoints(racename, races[currentrace]['positions'])
				log("raceEventHandler: created all checkpoints for "..racename)
				races[racename]['checkpointsloaded'] = true
			end
			flashMessage("Race "..racename.." started!", 4)
			--else
			-- passed some not-start checkpoint
			--flashMessage("This is checkpoint "..checknum.." of the Race "..racename.."!", 4)
		end
	elseif currentrace == racename then
		--log(checknum.."###"..races[currentrace]['numcheckpoints'].."###"..races[currentrace]['lastcheckpoint'].."###"..tostring(races[currentrace]['isloop']))
		--log("passed: "..races[currentrace]['checkpointspassed'])
		-- we hit a checkpoint from the same race!
		if checknum == races[currentrace]['lastcheckpoint'] + 1 then
			-- racing, correct checkpoint!
			if checknum == races[currentrace]['numcheckpoints'] and races[currentrace]['isloop'] then
				setupArrow(1)
			else
				setupArrow(checknum+1)
			end
			races[currentrace]['lastcheckpoint'] = checknum
			races[currentrace]['checkpointspassed'] = races[currentrace]['checkpointspassed'] + 1
			time = getTime() - races[currentrace]['startTime']
			msg = "Passed checkpoint "..checknum.." of "..(races[currentrace]['numcheckpoints']).." after "..string.format("%.3f", time).." seconds."
			flashMessage(msg, 4)
		else
			if not checknum == races[currentrace]['lastcheckpoint'] then
				-- racing, wrong checkpoint, prevent trigger of the same checkpoint twice
				flashMessage("Wrong gate! You must find and pass Checkpoint "..races[currentrace]['lastcheckpoint'] + 1, 4)
			end
		end
		if checknum == races[currentrace]['goalnum'] and races[currentrace]['checkpointspassed'] == races[racename]['numcheckpoints'] then
			-- racing, reached finish
			--time = getTime() - starttime
			time = stopTimer()
			msg = "Finished! You needed "..string.format("%.3f", time).." seconds!"

			if races[currentrace]['besttime'] == 9999 or time < tonumber(races[currentrace]['besttime']) then
				races[currentrace]['besttime'] = time
				writeScore(races[currentrace]['racename'], time)
				msg = msg .. "\nNew Best Lap Time!"
			end
			msg = msg .. "\n (Best Lap: "..string.format("%.3f", races[currentrace]['besttime'])..")"
			
			
			setupArrow(-1)
			-- reset some values
			races[currentrace]['checkpointspassed'] = 0
			
			currentrace = ""
			-- flash finish message for 10 seconds
			flashMessage(msg, 10)
		end
	else
		-- flashMessage("This Checkpoint belongs to race"..racename.."!", 4)
	end


end

truck_race = {
{103.289001, 8.481837, 282.813934, 0.0, 89.0, 0.0},
{122.295944, 7.034138, 211.191971, 0.0, 160.0, 0.0},
{197.703323, 8.146162, 149.602982, 0.0, 132.0, 0.0},
{295.123474, 12.093315, 83.150101, 0.0, 97.0, 0.0},
{378.637726, 12.466372, 99.757881, 0.0, 55.0, 0.0},
{344.430450, 13.978177, 163.362747, 0.0, -28.0, 0.0},
{401.885223, 13.160230, 181.328735, 0.0, -76.0, 0.0},
{470.363678, 9.839262, 188.886993, 0.0, -115.0, 0.0},
{553.223999, 11.979385, 231.670502, 0.0, -115.0, 0.0},
{614.428345, 10.338456, 261.922791, 0.0, -115.0, 0.0},
{707.690369, 11.329387, 209.888107, 0.0, -53.0, 0.0},
{840.095886, 15.041605, 142.723358, 0.0, -79.0, 0.0},
{842.982178, 13.909688, 169.207108, 0.0, -200.0, 0.0},
{716.213867, 10.361163, 323.596466, 0.0, -210.0},, 0.0},
{658.312073, 11.964308, 393.369415, 0.0, -210.0},, 0.0},
{588.562439, 10.124179, 491.178558, 0.0, -210.0},, 0.0},
{588.058105, 12.476987, 541.502686, 0.0, -120.0},, 0.0},
{721.685608, 13.057305, 623.736267, 0.0, -120.0},, 0.0},
{721.700745, 13.532433, 680.362061, 0.0, -190.0, 0.0},
{679.789185, 14.334210, 723.727417, 0.0, -230.0, 0.0},
{614.104797, 12.412940, 749.957458, 0.0, -256.0, 0.0},
{508.363953, 11.911936, 748.852051, 0.0, -275.0, 0.0},
{460.028564, 12.318174, 723.787415, 0.0, -275.0, 0.0},
{359.421539, 9.844972, 692.707947, 0.0, -275.0, 0.0}
{303.246552, 14.217538, 664.877502, 0.0, -335.0, 0.0},
}

setupRace("truckrace", truck_race, true, "chp-start", "chp-checkpoint", false)
------------------------------------------------------------------------------------------------------------

log("rdes.terrn.lua file loaded")

end
